/*******************************************************************************
 * Copyright (c) 2006 Novell, Inc..
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jim Sermersheim - Initial cut of interfaces agreed upon 20060707
 *******************************************************************************/

package org.eclipse.higgins.idas;

import java.net.URI;

/**
 * As defined at <a href="http://spwiki.editme.com/Context">http://spwiki.editme.com/Context</a> and <a href="http://spwiki.editme.com/ContextProvider">http://spwiki.editme.com/ContextProvider</a>, This interface holds a set of {@link IDigitalSubject} objects.<p>
 * Note that the {@link IHasRelationships} interface takes a {@link URI} type. This URI is a Context Referenece to the related {@link IContext}.
 * 
 * TODO (Arch): Should there be a removeSubject method which takes a filter? Similar notion could apply to other update operations as they are added.<p> 
 * 
 */
public interface IContext extends IHasMetadata, IHasRelationships<URI>
{
	/*
	 * Context-wide methods
	 */

	/**
	 * Opens this Context using the provided identity.<p>
	 * Upon success (return with no exception), this Context is in an "open" state 
	 * and is available to have other methods called (such as 
	 * {@link #getSubject(String)}, 
	 * {@link #getSubjects(IFilter)}, 
	 * {@link #createSubject(URI, Iterable)}, 
	 * {@link #removeSubject(String)}, 
	 * {@link #verifySubjectAttributes(String, Iterable)}, 
	 * {@link #updateSubject(String, Iterable)}, 
	 * {@link #exportData(String, String)}, 
	 * {@link #importData(String, String)},
	 * ). 
	 * Methods requiring the context to be open throw {@link ContextNotOpenException} 
	 * when in an un-open state.<p>
	 * An open state specifies that the authentication materials in the identity argument have been used to successfully authenticate to this Context.<p>
	 * If this IContext is already in an open state, this throws {@link ContextOpenException}.<p>
	 * The antithesis of this method is close().
	 *
	 * @param identity AuthN identity used to open the context.
	 * @throws IdASException
	 * @throws ContextOpenException
	 * @see #close()
	 */
	public void	open(Object identity) throws IdASException, ContextOpenException;
	
	/**
	 * Closes this Context and returns it to an un-open state.<p>
	 * Methods requiring the context to be open throw ContextNotOpenException 
	 * when in an un-open state.<p>
	 * 
	 * @throws IdASException
	 */
	public void	close() throws IdASException;

	/**
	 * Tests whether this context is open and authenticated using the specified identity
	 * @param identity 	May be {@code null}. 
	 * 					When provided, asserts that this context was opened with the specified identity. 
	 * 					When (@code null), no identity check is performed. 
	 * @return {@code true} if this context is open as specified, {@code false} otherwise
	 * @throws IdASException
	 */
	public boolean isOpen(Object identity) throws IdASException;
	
	/**
	 * Returns the schema governing this context.<p>
	 * TODO (Doc): Add pointers to OWL.<p>
	 * TODO (Doc): Add (or point to) documentation which includes sample schema.<p>
	 * TODO (Arch)(Impl): Add method(s) which return the schema as a stream, DOM document, or other iterable object.
	 * 
	 * @return schema is returned as a {@link String} in OWL-DL format.
	 * @throws IdASException
	 */
	public String getSchema() throws IdASException;

	/**
	 * Returns a reference to this Context. This value is used in Context Relationships and used to open this Context.
	 * 
	 * @return the reference to this Context.
	 * @throws IdASException
	 */
	public URI getContextRef() throws IdASException;

	/**
	 * Exports this Context.<p>
	 * TODO (Arch): filter format TBD<p>
	 * TODO (Arch): Does this export only Digital Subjects, or the entire Context (Metadata, Schema, DS, Relationships)?
	 * @param filter May be {@code null}. Used to select a subset of this Context to be exported.
	 * @param representationFormat Indicates the format of the string to be returned. TODO (Arch)(Doc): Specify format type. Give example
	 * @return String in the form of representationFormat containing the exported Context.
	 * @throws IdASException
	 */
	public String exportData(String filter, String representationFormat) throws IdASException;

	/**
	 * Imports this Context.<p>
	 * TODO (Arch): filter format TBD<p>
	 * TODO (Arch): Does this import only Digital Subjects, or the entire Context (Metadata, Schema, DS, Relationships)?
	 * TODO (Arch)(Doc): Discuss collisions.
	 * @param filter May be {@code null}. Used to select a subset of the external data set to be imported.
	 * @param representationFormat Indicates the format of the external data string being specified in external. TODO (Arch)(Doc): Specify format type. Give example
	 * @throws IdASException
	 */
	public void	importData(String filter, String representationFormat) throws IdASException;

	
	/*
	 * Methods relating to Digital Subjects
	 */

	/**
	 * Creates an IFilter instance that may be used when calling {@link #getSubjects(IFilter)}
	 */
	public IFilter createFilter() throws IdASException;
	
	/**
	 * Returns the Digital Subject matching the specified cuid.
	 * This is effectively the same as calling {@link #getSubjects} with
	 * the filter set to {@literal <appropriate filter for cuid=name>}<p>
	 * TODO (Arch): we need a way to present a properties constraint. This is to help providers which are gathering their data from expensive sources (i.e. ldap server). We don't want to cause multiple network hits, or extra data to be gathered.<p>
	 * TODO (Arch)(Doc): discuss whether modifications to the returned IDigitalSubject affect the stored subject.<p>
	 * 
	 * @param cuid Contextually Unique ID of Digital Subject to return.<p>
	 * 				
	 * @return IDigitalSubject.
	 * @throws IdASException (typically a subclass of. See below).
	 * @throws NoSuchSubjectException when subject specified by cuid is not found.
	 *
	 */
	public IDigitalSubject getSubject(String cuid) throws IdASException;
	
	/**
	 * From this context, return {@link IDigitalSubject}s that match the specified filter <p>
	 * TODO (Arch): define filter format.<p>
	 * TODO (Arch): add param: Set<URI> attributeSelection which is "The attributes on IDigitalSubject objects which will be read by the caller. An empty set or null indicates that all attributes will be read."<p>
	 * @param filter 	Filter used to constrain the set of {@link IDigitalSubject}s returned.
	 * 					A {@code null} or empty filter causes all {@link IDigitalSubject}s to be returned.
	 * @return An {@link Iterable} set of {@link IDigitalSubject}s
	 * @throws IdASException
	 *
	 */
	public Iterable<IDigitalSubject> getSubjects(IFilter filter) throws IdASException;
	
	/**
	 * Creates and adds a Digital Subject to this Context.<p>
	 * 
	 * @param subjectType The type of Digital Subject being created. TODO (Doc): add example URI to a higgins person.
	 * @param attributes The attributes of the Digital Subject being created. Attributes required by the definition of the subjectType must be included.  
	 * @return The cuid of the Digital Subject which was added as a result of this operation. 
	 * @throws IdASException
	 */
	public String createSubject(URI subjectType, Iterable<IAttribute> attributes) throws IdASException;

	/**
	 * Removes a Digital Subject from this Context. 
	 * 
	 * @param cuid Contextually Unique ID of the Digital Subject to be removed from this Context.<p>
	 * @throws IdASException (typically a subclass of. See below).
	 * @throws NoSuchSubjectException when subject specified by cuid is not found.
	 */
	public void removeSubject(String cuid) throws IdASException;

	/**
	 * Simple method for comparing a set of attribute assertions against a Digital Subject.<p>
	 * TODO (Arch): This doesn't leave a way to verify that a DS representing a group contains a specific member in its member value.
	 * 		We may want to consider allowing a filter to be passed here, or at least a partial attribute.
	 * 
	 * @param cuid Contextually Unique ID of Digital Subject being verified.
	 * @param attributes Set of attributes to be compared to the Digital Subject specified by cuid. 
	 * @return {@code true} if all attribute assertions in attributes are found to be equal to those attributes in the Digital Subject specified by cuid. {@code false} otherwise. 
	 * @throws IdASException (typically a subclass of. See below).
	 * @throws NoSuchSubjectException when subject specified by cuid is not found.
	 */
	public boolean verifySubjectAttributes(String cuid, Iterable<IAttribute> attributes) throws IdASException;

	/**
	 * Add or change the attributes of a Digital Subject.<p>
	 * TODO (Arch): This doesn't allow deletion of an attribute.
	 * TODO (Arch): This doesn't allow update operations on attribute parts (such as adding a single member of a member value)
	 * TODO (Arch)(Doc): Discuss transactional semantics. Is this guaranteed to fully succeed or fully fail?
	 * 
	 * @param cuid Contextually Unique ID of Digital Subject being updated.<p>
	 * @param attributes {@link Iterable} set of attributes to be added or replaced.
	 * @throws IdASException
	 */
	public void	updateSubject(String cuid, Iterable<IAttribute> attributes) throws IdASException;

}
