/*******************************************************************************
 * Copyright (c) 2006 Novell, Inc..
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jim Sermersheim - Initial cut
 *******************************************************************************/

package org.eclipse.higgins.idas.impl;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.higgins.idas.IPropertyValue;
import org.eclipse.higgins.idas.ISimpleValue;
import org.eclipse.higgins.idas.IdASException;


public class BasicValueNormalizedString extends BasicValueString implements ISimpleValue {
	
	public static final String XML_TYPE_STR = "normalizedString";
	public static final String OWL_TYPE_STR = XML_TYPE_STR + AbstractSimpleValue.OWL_TYPE_SUFFIX;
	public static final String ATTR_VALUE_URI_STR = IPropertyValue.BASE_OWL_URI + OWL_TYPE_STR; 
	public static final String ATTR_VALUE_TYPE_URI_STR = ISimpleValue.BASE_XML_SCHEMA_URI + XML_TYPE_STR;

	protected enum WhiteSpaceRule {preserve, replace, collapse};
	
	/**
	 * Constructs using a non-normalized string
	 * @param value Non-normalized string.
	 */
	public BasicValueNormalizedString(String value) {
		_setValue(_normalize(value, WhiteSpaceRule.replace));
	}

	/**
	 * Converts a string to a normalized string.
	 * Converts all tabs, line feeds, and carriage returns to spaces (http://www.w3.org/TR/xmlschema-2/#normalizedString)
	 * Converts occurrence of carriage return + line feed to a single space ( http://www.w3.org/TR/REC-xml/#sec-line-ends)
	 * 
	 * @param value String to be normalized
	 * @param rule whiteSpace rule to be followed (http://www.w3.org/TR/xmlschema-2/#rf-whiteSpace)
	 * @return the normalized string
	 */
	protected String _normalize(String value, WhiteSpaceRule rule)
	{
		if (rule == WhiteSpaceRule.preserve) {
			return value;
		}

		int	iChar = 0,
			iLen = value.length();
		StringBuffer normVal = new StringBuffer(iLen);
		char c;
		
		for (; iChar < iLen; iChar++) {
			c = value.charAt(iChar);
			if (c != 0x09 && c != 0x0D && c != 0x0A) {
				normVal.append(c);
			}
			else {
				// Tread 0x0D,0x0A as a single space
				if ((c == 0x0D) && (value.charAt(iChar + 1) == 0x0A)) {
					iChar++;
				}
				if (rule == WhiteSpaceRule.collapse) {
					// No initial space
					if (normVal.length() == 0) {
						continue;
					}
					// Only single space
					if (normVal.charAt(normVal.length() - 1) == 0x20) {
						continue;
					}
				}
				normVal.append(' ');
			} 
		}
		// No trailing space
		if ((rule == WhiteSpaceRule.collapse) && (normVal.charAt(normVal.length() - 1) == 0x20)) {
			normVal.deleteCharAt(normVal.length() - 1);
		}
		return normVal.toString();
	}

	/**
	 * Returns a URI for {@value #ATTR_VALUE_TYPE_URI_STR}
	 */
	public URI getType() throws IdASException {
		try {
			return new URI(ATTR_VALUE_TYPE_URI_STR);
		} catch (URISyntaxException e) {
			throw new IdASException(e);
		}
	}
}
